﻿using FellowOakDicom.Network.Client.EventArguments;
using FellowOakDicom.Network;
using FellowOakDicom;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FellowOakDicom.Network.Client;
using System.Threading;

namespace DicomCfindSample
{
    public class Program
    {
        // Dr. Harvey graciously provides a free DICOM server to connect and play with
        private static string QRServerHost = "www.dicomserver.co.uk";
        private static int QRServerPort = 104;
        private static string QRServerAET = "STORESCP";
        private static string AET = "FODICOMSCU";

        static async Task Main(string[] args)
        {
            try
            {
                // Создаем клиент для установления ассоциации
                var client = DicomClientFactory.Create(QRServerHost, QRServerPort, false, AET, QRServerAET);
                client.NegotiateAsyncOps();

                //Добавляем обработчик, который будет получать уведомления о любых отклонениях ассоциации.
                client.AssociationRejected += ClientOnAssociationRejected;

                //Добавляем обработчик для получения уведомлений о любой информации об ассоциации при успешных соединениях.
                client.AssociationAccepted += ClientOnAssociationAccepted;

                //Добавляем обработчик для уведомления об успешном освобождении ассоциации — это также может быть вызвано удаленным узлом.
                client.AssociationReleased += ClientOnAssociationReleased;

                // Создаём запрос на поиск исследований пациента с фамилией "Bowen", * - заменяет любое количество любых других символов
                DicomCFindRequest studyRequest = StudyCFindRequest("Bowen*");

                // Добавляем обработчик ответа на запрос - вывод результатов в консоль
                DicomCFindRequest.ResponseDelegate cFindResponseDelegate = LogStudyResultsFoundToDebugConsole;
                studyRequest.OnResponseReceived = cFindResponseDelegate;

                // Добавляем запрос на поиск исследований пациента 
                await client.AddRequestAsync(studyRequest);

                // Устанавливаем время ожидания, по истечении которого ассоциация будет разорвана принудительно  - 30 секунд
                CancellationTokenSource cancelToken = new CancellationTokenSource();
                CancellationToken TokenCancel = cancelToken.Token;
                DicomClientCancellationMode cancelMode = DicomClientCancellationMode.ImmediatelyReleaseAssociation;
                int timeToken = 30000;
                cancelToken.CancelAfter(timeToken);

                await client.SendAsync(TokenCancel, cancelMode); // Собственно посылка запроса с ожиданием результатов
                Console.ReadLine();
            }
            catch (Exception e)
            {
                //In real life, do something about this exception
                LogToDebugConsole($"Error occured during DICOM association request -> {e.StackTrace}");
            }            
        }
        public static DicomCFindRequest StudyCFindRequest(string patientName)
        {
            // Создание нового экземпляра запоса уровня "Study Root"
            var request = new DicomCFindRequest(DicomQueryRetrieveLevel.Study);

            // Чтобы получить интересующие атрибуты нужно указать их заранее с пустыми параметрами, как показано ниже
            request.Dataset.AddOrUpdate(DicomTag.PatientName, string.Empty);
            request.Dataset.AddOrUpdate(DicomTag.PatientID, string.Empty);
            request.Dataset.AddOrUpdate(DicomTag.StudyDate, string.Empty);
            request.Dataset.AddOrUpdate(DicomTag.StudyInstanceUID, string.Empty);

            // Фильтр по имени пациента 
            request.Dataset.AddOrUpdate(DicomTag.PatientName, patientName);

            // Specify the encoding of the retrieved results
            // here the character set is 'Latin alphabet No. 1'
            request.Dataset.AddOrUpdate(DicomTag.SpecificCharacterSet, "ISO_IR 100");                    
            return request;
        }
        private static void LogStudyResultsFoundToDebugConsole(DicomCFindRequest request, DicomCFindResponse response)
        {
            // Пока не получен сигнал о выполнении извлекаем из ответа значения тегов и выводим в консоль
            if (response.Status.ToString() == "Success")
            {
                LogToDebugConsole("The search is over...");
            }
            else
            {
                var patientName = response.Dataset.GetSingleValueOrDefault(DicomTag.PatientName, string.Empty);
                var patientID = response.Dataset.GetSingleValueOrDefault(DicomTag.PatientID, string.Empty);
                var studyDate = response.Dataset.GetSingleValueOrDefault(DicomTag.StudyDate, new DateTime());
                var studyUID = response.Dataset.GetSingleValueOrDefault(DicomTag.StudyInstanceUID, string.Empty);

                LogToDebugConsole("Matched Result...");
                LogToDebugConsole($"Patient Found ->  {patientName} ");
                LogToDebugConsole($"Patient ID ->  {patientID} ");
                LogToDebugConsole($"Study Date ->  {studyDate} ");
                LogToDebugConsole($"Study UID ->  {studyUID} ");
                LogToDebugConsole("\n");
            }
        }        
        private static void ClientOnAssociationReleased(object sender, EventArgs e)
        {
            LogToDebugConsole("Assoacition was released");           
        }
        private static void ClientOnAssociationRejected(object sender, AssociationRejectedEventArgs e)
        {
            LogToDebugConsole($"Association was rejected. Rejected Reason:{e.Reason}");            
        }
        private static void ClientOnAssociationAccepted(object sender, AssociationAcceptedEventArgs e)
        {
            var association = e.Association;
            LogToDebugConsole($"Association was accepted by remote host: {association.RemoteHost} running on port: {association.RemotePort}");

            foreach (var presentationContext in association.PresentationContexts)
            {
                if (presentationContext.Result == DicomPresentationContextResult.Accept)
                {
                    LogToDebugConsole($"\t {presentationContext.AbstractSyntax} was accepted");
                    LogToDebugConsole($"\t Negotiation result was: {presentationContext.GetResultDescription()}");
                    LogToDebugConsole($"\t Abstract syntax accepted: {presentationContext.AbstractSyntax}");
                    LogToDebugConsole($"\t Transfer syntax accepted: {presentationContext.AcceptedTransferSyntax.ToString()}");
                }
                else
                {
                    LogToDebugConsole($"\t Presentation context with proposed abstract syntax of '{presentationContext.AbstractSyntax}' was not accepted");
                    LogToDebugConsole($"\t Reject reason was {presentationContext.GetResultDescription()}");
                }
            }           
        }
        private static void LogToDebugConsole(string informationToLog)
        {
            Console.WriteLine(informationToLog);
        }

    }
}
